﻿#light
#r @"FSharp.PowerPack";;

#load "lexer.fs";;

open System
open Microsoft.FSharp.Text.Lexing
open Microsoft.FSharp.NativeInterop
open Lexert

//a jelenlegi folyó adatbázis
let rivers = [("Nilus", 6690);
              ("Mississipi", 6200);
              ("Jangce", 5800);
              ("Duna", 2842);
              ("Amur", 5780);
              ("Don", 1870);
              ("Amazonas", 6500);]
 
let parse lexbuf =

    let mutable keepParsing = true

    let mutable tokenList = []

    

    while keepParsing = true do

      
 

        // 'parsetokens' a metódus amit az fslex generál

        let parsedToken = Lexert.parsetokens lexbuf

        // adjuk a tokeneket a listánkhoz

        tokenList <- tokenList @ [parsedToken]

    

        if parsedToken = Tokens.EOF then
        //olvassunk míg vége nincs a bemenetnek
            keepParsing <- false

            

    tokenList;;

 

// megpróbáljuk beolvasni a szövegblokkot és kiírni a státuszokat a standard outputra

let tryParse text =

    let bytes = System.Text.Encoding.ASCII.GetBytes (text : string)
    let lexbuf = LexBuffer.FromBytes bytes 
    
      
    try
        //a kérdés értelmezéséhez ugyan minden token kell, de a megfelelő adatbázis művelet végrehajtásához nem
        let need_tokens token  =
            match token with
                |Tokens.HB -> true
                |Tokens.LH -> true
                |Tokens.LR -> true
                |Tokens.RB -> true
                |Tokens.FOLYO folyo ->true
                |_ -> false
        
        //az előállított byte buffert futtassuk végig a lexeren, próbáljuk értelmezni, ha sikerül akkor kapunk egy tokenlistát        
        let tokens = parse lexbuf |> List.filter (fun x -> need_tokens x)//szűrjük le a kapott tokeneket
        
        //folyónév alapján keressük meg a tényleges párt, ami tartalmazza a hosszt (fst(x) hivatkozik a pár első értékére, snd(x) a másodikra)
        let get_reference river = rivers|>List.tryFind(fun x -> match x with 
                                                                    |_ when fst(x)=river->true
                                                                    |_,_->false)
        let river_data river = get_reference river

        //keresünk egy hosszabb folyót, a folyó listán végig iterálva
        //mintaillesztéssel ellenőrizzük, hogy a lekérdezett folyó szerepel e az adatbázisban (ha igen, érvényes értékünk van egy pár formájában, ha nem, akkor None)
        //viszont None érték esetén csak az utolsó elem vizsgálatakor írunk ki hibaüzenetet
        let list_longer river   =  rivers |>List.iter(fun  x ->match river with
                                                                | Some(j,k) when k<snd(x) -> 
                                                                    printfn "%s(%d) hosszab mint a(z) %s(%d)" (fst(x))(snd(x)) j k
                                                                | None when x = rivers.[rivers.Length-1] ->   printfn "%A  nincs az adatbázisban" river
                                                                | _ -> printf "")
        
        //keresünk egy rövidebb folyót
        let list_shorter river   =  rivers |>List.iter(fun  x ->match river with
                                                                | Some(j,k) when k>snd(x) -> 
                                                                    printfn "%s(%d) rövidebb mint a(z) %s(%d)" (fst(x))(snd(x)) j k
                                                                | None when x = rivers.[rivers.Length-1] -> printfn "%A  nincs az adatbázisban" river
                                                                | _ -> printf "")
        
        //keressük a legrövidebb folyót 
        //segítségünkre van a Seq osztály metódusa, az snd jelöli, hogy a pár második értéke alapján keressen                                                     
        let list_shortest   =  rivers |>Seq.minBy snd
        
        //keressük a leghosszabb folyót        
        let list_longest   =  rivers |>Seq.maxBy snd
        
        //itt adjuk meg, hogy milyen tokenek milyen sorrendje esetén milyen műveletet hajtsunk végre
        let filtered tokens = match tokens with
                                |[Tokens.HB;Tokens.FOLYO folyo]-> list_longer (get_reference folyo)
                                |[Tokens.RB;Tokens.FOLYO folyo]-> list_shorter (get_reference folyo)
                                |[Tokens.LR;]-> printfn "%A  a legrövidebb folyó" list_shortest
                                |[Tokens.LH;]-> printfn "%A  a leghosszabb folyó" list_longest
                                |_-> printfn "Nem értem"
               
        filtered tokens

    with e -> 

        let pos = lexbuf.EndPos

        printfn "Exception Message: %s\nLex error near line %d, character %d" 

                e.Message pos.Line pos.Column;;

//belépési pont
[<EntryPoint>]
let main args =
    let mutable run = true
    printfn "Ez a program folyók hosszának lekérdezésére alkalmas."
    printfn "Minden kérdés után a program újabb kérdést vár, amíg az 'exit' szót nem írjuk be."
    printfn "A kérdéseket több formában is megadhatjuk, pl 'Melyik a leghosszabb folyo' vagy 'Melyik folyo hosszabb mint a Duna'."
    printfn "A program kis és nagy betűkre érzékeny (nyelvtanilag helyes kérdést vár) és nem kezeli az ékezeteket!"

    while run do
        //újabb kérdéseket dolgozunk fel míg exit-et nem kapunk
        printfn "Kérem a kérdést:"
        let param = System.Console.ReadLine()
        if param = "exit" then
            run <- false
        else tryParse param

    printfn "Viszlát!"
    0